package org.amse.byBedrosova.go.io;

import java.io.IOException;
import java.io.InputStream;

import javax.swing.JFrame;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.*;
import org.xml.sax.SAXException;

import org.amse.byBedrosova.go.model.*;
import org.amse.byBedrosova.go.exception.*;
import org.amse.byBedrosova.go.view.*;

/**
 * ,      XML- 
 */
public class GoReader {
	private JFrame myView;
	private IGame myGame;
	
	public GoReader (IGame game) {
		myGame = game;
	}
	
	public GoReader (View view) {
		myView = view;
	}
	
	private Document parseDocument (String fileName) {
		DocumentBuilder b;

		try {
			b = DocumentBuilderFactory.newInstance().newDocumentBuilder();
		} catch (ParserConfigurationException e) {
			throw (new GoException ("Creating document builder failed"));
		}
		Document d;
		
		try {
			d = b.parse(fileName);		
		} catch (SAXException e) {
			throw (new GoException ());
		} catch (IOException e) {
			throw (new GoException ("Parsing file failed"));			
		}	
		return d;
	}
	
	
	public void loadGame (String fileName) {
		Document d = parseDocument(fileName);
		processGame (d.getDocumentElement());
	}

	
	private void processGame (Node node) {
		if (!node.getNodeName().equals("game") || node.getNodeType() != Node.ELEMENT_NODE ||
			!node.hasChildNodes()) {
			throw new GoException ("Wrong root element in game");
		}
		NodeList moveElements = ((Element)node).getElementsByTagName("move");
		
		NamedNodeMap [] moves = new NamedNodeMap [moveElements.getLength()];
		
		for (int i = 0; i < moveElements.getLength(); ++i) {
			processMove (moveElements.item(i), moves);
		} 
		try {
			for (int i = 0; i < moves.length; ++i) {
				NamedNodeMap attributes = moves [i];
				
				if (!attributes.getNamedItem("color").getNodeValue().equals(myGame.getCurrentPlayer().toString())) {
					throw new GoException ("Wrong move color in file");		
				}
				Node tmpPassed = attributes.getNamedItem("passed");			
				
				if ((tmpPassed != null) && tmpPassed.getNodeValue().equals("true")) {
					if (myGame.canPass()) {
						myGame.pass();
					} else {
						throw new GoException ("Wrong move in file");					
					}
				} else {
					int vertical = Integer.parseInt(attributes.getNamedItem("vertical").getNodeValue());
					int horizontal = Integer.parseInt(attributes.getNamedItem("horizontal").getNodeValue());
					
					if (myGame.canDoMove(vertical, horizontal)) {
						myGame.doMove(vertical, horizontal);
					} else {
						throw new GoException ("Wrong move in file");								
					}
				}
			}
			int tmpBufferCurrentIndex = Integer.parseInt (node.getAttributes().getNamedItem("bufferCurrentIndex").getNodeValue());
			
			for (int i = myGame.getBufferCurrentIndex(); i > tmpBufferCurrentIndex; --i) {
				if (myGame.canUndoMove()) {
					myGame.undoMove();
				} else {
					throw new GoException ("Index error");
				}
			}
		} catch (NullPointerException ex) {
			throw new GoException ("Wrong attribute in file");			
		} catch (NumberFormatException ex) {
			throw new GoException ("Wrong attribute in file");								
		}
	}
	
	private void processMove (Node node, NamedNodeMap [] moves) {
		try {
			if (!node.getNodeName().equals("move") || !node.hasAttributes() || node.getNodeType() != Node.ELEMENT_NODE) {
				throw new GoException ("Wrong move element");
			}			
			NamedNodeMap tmp = node.getAttributes();			
			
			int index = Integer.parseInt(tmp.getNamedItem("index").getNodeValue());			
			
			if (moves [index] != null) {
				throw new GoException ("Duplicate move index");
			}
			moves [index] = tmp;
		} catch (NullPointerException ex) {
			throw new GoException ("Missing move index attribute");
		} catch (ArrayIndexOutOfBoundsException ex) {
			throw new GoException ("Some index missed");
		} catch (NumberFormatException ex) {
			throw new GoException ("Wrong attribute value");
		}
	}
	
	public void loadSettings (String fileName) {
		Document d = parseDocument(fileName);	
		processSettings(d.getDocumentElement());
	}
	
	private void processSettings (Node node) {
		if (!node.getNodeName().equals("settings") || node.getNodeType() != Node.ELEMENT_NODE ||
			!node.hasAttributes()) {
			throw new GoException ("Wrong root element in settings");
		}
		NamedNodeMap attributes = node.getAttributes();
		try {
			int horizontal = Integer.parseInt (attributes.getNamedItem("horizontal").getNodeValue());
			int vertical = Integer.parseInt (attributes.getNamedItem("vertical").getNodeValue());
			myView.setLocation (horizontal, vertical);
			
			Node attribute = attributes.getNamedItem("showBoardNumbers");
			if (attribute != null) {
				((View)myView).getPanel ().showNumbers(Boolean.parseBoolean(attribute.getNodeValue()));
			} else {
				((View)myView).getPanel ().showNumbers(true);
			}
			attribute = attributes.getNamedItem("showToolBar");
			if (attribute != null) {
				((View)myView).getToolBar().setVisible(Boolean.parseBoolean(attribute.getNodeValue()));				
			} else {
				((View)myView).getToolBar().setVisible(true);
			}
			attribute = attributes.getNamedItem("lightUpSurrounded");
			if (attribute != null) {
				((View)myView).getPanel ().lightUpSurrounded(Boolean.parseBoolean(attribute.getNodeValue()));
			} else {
				((View)myView).getPanel ().lightUpSurrounded(true);
			}	
		} catch (NullPointerException ex) {
			throw new GoException ("Missing coordinate attribute");			
		} catch (NumberFormatException ex) {
			throw new GoException ("Wrong attribute value");			
		}
	}
	
	public String loadAbout (String fileName) {
		DocumentBuilder b;

		try {
			b = DocumentBuilderFactory.newInstance().newDocumentBuilder();
		} catch (ParserConfigurationException e) {
			throw (new GoException ("Creating document builder failed"));
		}
		InputStream is = View.class.getClassLoader().getResourceAsStream(fileName);
		Document d;		
		try {
			d = b.parse(is);		
		} catch (SAXException e) {
			throw (new GoException ("Parsing file failed"));
		} catch (IOException e) {
			throw (new GoException ("Parsing file failed"));			
		} catch (IllegalArgumentException e) {
			throw (new GoException ("Wrong file"));			
		}
		Node node = d.getDocumentElement();
		if (!node.getNodeName().equals("about") || node.getNodeType() != Node.ELEMENT_NODE) {
				throw new GoException ("Wrong root element in settings");
		}		
		return node.getTextContent(); 
	}
		
}
