package org.amse.byBedrosova.go.io;

import javax.swing.JFrame;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.*;

import java.io.*;

import java.util.List;

import org.amse.byBedrosova.go.view.*;
import org.amse.byBedrosova.go.model.*;
import org.amse.byBedrosova.go.exception.*;

/**
 * ,          XML- 
 */
public class GoWriter {
	private JFrame myView;
	private IGame myGame;
	
	public GoWriter (View view) {
		myView = view;
	}
	
	public GoWriter (IGame game) {
		myGame = game;
	}
	
	private Document createDocument () {
		DocumentBuilder b;

		try {
			b = DocumentBuilderFactory.newInstance().newDocumentBuilder();
		} catch (ParserConfigurationException e) {
			throw new GoException ("Creating document builder failed");
		}		
		return b.newDocument();		
	}
	
	public void saveGame (String fileName) {		
		Document d = createDocument();
		d.appendChild(createGameElement(d));		
		transform(d, fileName);		
	}	
	
	private Element createGameElement (Document d) {
		Element gameElement = d.createElement("game");
		gameElement.setAttribute("bufferCurrentIndex", "" + myGame.getBufferCurrentIndex());

		List<IMoveRecord> tmpBuffer = myGame.getGameBuffer();
		
		int i = 0;
		for (IMoveRecord record : tmpBuffer) {
			Element recordElement = d.createElement("move");			
			
			recordElement.setAttribute("index", "" + i++);
			
			if (record.isPass()) {
				recordElement.setAttribute("passed", "true");
			} else {
				recordElement.setAttribute("horizontal", "" + record.getStone().getHorizontal());
				recordElement.setAttribute("vertical", "" + record.getStone().getVertical());
			}
			recordElement.setAttribute("color", record.getColor().toString());
			
			gameElement.appendChild(recordElement);			
		}
		return gameElement;
	}
	
	public void saveViewSettings (String fileName) {
		Document d = createDocument();		
		d.appendChild(createSettingsElement (d));		
		transform(d, fileName);
	}
	
	private Element createSettingsElement (Document d) {
		Element settingsElement = d.createElement("settings");		
		
		java.awt.Point tmpLocation = myView.getLocation();
		settingsElement.setAttribute("horizontal", new Integer ((int)tmpLocation.getX()).toString());
		settingsElement.setAttribute("vertical", new Integer ((int)tmpLocation.getY ()).toString());
		if (!((View)myView).getToolBar().isVisible()) {
			settingsElement.setAttribute("showToolBar", "false");			
		}
		if (!((View)myView).getPanel().areNumbersShown()) {
			settingsElement.setAttribute("showBoardNumbers", "false");			
		}
		if (!((View)myView).getPanel().areSurroundedLitUp()) {
			settingsElement.setAttribute("lightUpSurrounded", "false");			
		}		
		return settingsElement;
	}

	private void transform (Document d, String fileName) {
		TransformerFactory tf;
		tf = TransformerFactory.newInstance();
		tf.setAttribute("indent-number", 4);
		
		Transformer t;
		
		try {
			t = tf.newTransformer();
			t.setOutputProperty(OutputKeys.INDENT, "yes");
			t.transform(new DOMSource (d), new StreamResult (new FileWriter (new File (fileName))));
		} catch (TransformerConfigurationException ex) {
			throw new GoException ("Transforming failed");
		} catch (TransformerException ex) {
			throw new GoException ("Transforming failed");
		} catch (IOException ex) {
			throw new GoException ("Writing to file failed");
		}		
	}
}
