package org.amse.byBedrosova.go.model.impl;

import java.util.Collections;
import java.util.List;
import java.util.ArrayList;
import java.util.Set;

import org.amse.byBedrosova.go.exception.GoException;
import org.amse.byBedrosova.go.model.*;

class Game implements IGame {
	
	private IBoard myBoard;
	private GoColor myCurrentPlayer;
	
	private List<IMoveRecord> myGameBuffer;
	private int myBufferCurrentIndex;
	
	private boolean myModified;	
	
	private long myWhitesScores;
	private long myBlacksScores;
	
	public Game (int size) {
		myBoard = new Board (size);
		myCurrentPlayer = GoColor.BLACK;
		myGameBuffer = new ArrayList<IMoveRecord> ();
		myBufferCurrentIndex = 0;
		myModified = false;
		myWhitesScores = 0;
		myBlacksScores = 0;
	}
	
//	-------------------------------------------------------------------------
	public IBoard getBoard () {
		return myBoard;
	}	
//	-------------------------------------------------------------------------
	public GoColor getCurrentPlayer () {
		return myCurrentPlayer;
	}	
//	-------------------------------------------------------------------------
	public void changeCurrentPlayer () {
		myCurrentPlayer = myCurrentPlayer.getOppositeColor ();
	}
//	-------------------------------------------------------------------------
	public void setCurrentPlayer (GoColor color) {
		myCurrentPlayer = color;
	}	
//	-------------------------------------------------------------------------
	private IGroup getGroup (int vertical, int horizontal) {
		if ((myBoard.getStone(vertical, horizontal) != null) && 
			(myBoard.getStone(vertical, horizontal).getColor() != myCurrentPlayer)) {
			return myBoard.collectGroup(myBoard.getStone(vertical, horizontal));
		}
		return null;
	}	
	
	public boolean canDoMove (int vertical, int horizontal) {
		if (isGameOver() || (myBoard.getStone(vertical, horizontal) != null)) {
			return false;
		}		
		IGroup group = myBoard.collectGroup(vertical, horizontal, myCurrentPlayer);
		
		if (!group.isSurrounded()) {
			return true;
		}		
		if (ko (vertical, horizontal)) {
			return false;
		}
		if ((group = getGroup (vertical - 1, horizontal)) != null) {
			if (myBoard.isGroupSurrounded(group, vertical, horizontal)) {
					return true;
			}
		}	
		if ((group = getGroup (vertical, horizontal + 1)) != null) {
			if (myBoard.isGroupSurrounded(group, vertical, horizontal)) {
					return true;
			}
		}		
		if ((group = getGroup (vertical + 1, horizontal)) != null) {
			if (myBoard.isGroupSurrounded(group, vertical, horizontal)) {
					return true;
			}
		}
		if ((group = getGroup (vertical, horizontal - 1)) != null) {
			if (myBoard.isGroupSurrounded(group, vertical, horizontal)) {
					return true;
			}
		}
		return false;
	}
//	-------------------------------------------------------------------------
	private boolean ko (int vertical, int horizontal) {
		if (myGameBuffer.isEmpty()) {
			return false;
		}		
		IMoveRecord tmp  = myGameBuffer.get(myBufferCurrentIndex - 1);
		
		if (tmp.isPass()) {
			return false;
		}	
		List<IStone> killed = tmp.getKilled();
		for (IStone killedStone : killed) {
			if ((killedStone.getVertical() == vertical) && (killedStone.getHorizontal() == horizontal)) {
				return true;
			}
		}
		return false;
	}
//	-------------------------------------------------------------------------
	private Set<IStone> getSurroundedStones (int vertical, int horizontal) {
		if (myBoard.getStone(vertical, horizontal) != null) {
			if (myBoard.getStone(vertical, horizontal).getColor() == myCurrentPlayer) {
				IGroup group = myBoard.collectGroup(myBoard.getStone(vertical, horizontal));
				if (myBoard.isGroupSurrounded (group)) {
					myBoard.removeGroup(group);
					return group.stones();
				}
			}
		}
		return Collections.<IStone>emptySet();
	}
	
	public void doMove (int vertical, int horizontal) {
		if (!canDoMove(vertical, horizontal)) {
			throw new GoException ("Can not doMove");
		}
		myBoard.setStone(vertical, horizontal, myCurrentPlayer);
		while (myBufferCurrentIndex < myGameBuffer.size()) {
			myGameBuffer.remove(myGameBuffer.size () - 1);
		}
		List<IStone> tmp = new ArrayList<IStone> ();
		changeCurrentPlayer ();		

		tmp.addAll(getSurroundedStones (vertical - 1, horizontal));
		tmp.addAll(getSurroundedStones (vertical    , horizontal + 1));
		tmp.addAll(getSurroundedStones (vertical + 1, horizontal));
		tmp.addAll(getSurroundedStones (vertical    , horizontal - 1));
		
		myGameBuffer.add(new RealMoveRecord (myBoard.getStone(vertical, horizontal), tmp));
		incScores (myCurrentPlayer.getOppositeColor (), tmp.size());
		++myBufferCurrentIndex;
		myModified = true;
	}
	
//	-------------------------------------------------------------------------	
	public boolean canPass() {
		return myBufferCurrentIndex > 0 && !isGameOver();
	}
//	-------------------------------------------------------------------------	
	public void pass() {
		if (!canPass()) {
			throw new GoException ("Can not pass");
		}
		while (myBufferCurrentIndex < myGameBuffer.size()) {
			myGameBuffer.remove(myGameBuffer.size () - 1);
		}
		myGameBuffer.add(new PassMoveRecord(myCurrentPlayer));
		changeCurrentPlayer();
		++myBufferCurrentIndex;
		myModified = true;
	}
//	-------------------------------------------------------------------------
	public boolean canUndoMove () {
		return myBufferCurrentIndex > 0;
	}	
//	-------------------------------------------------------------------------
	public void undoMove () {
		if (!canUndoMove()) {
			throw new GoException ("Can not undoMove");
		}
		IMoveRecord tmp  = myGameBuffer.get(--myBufferCurrentIndex);

		if (!tmp.isPass()) {
			myBoard.removeStone(tmp.getStone());
	
			for (IStone stone : tmp.getKilled()) {
				myBoard.setStone(stone.getVertical(), stone.getHorizontal(), stone.getColor());
			}
			incScores (tmp.getStone().getColor(), - tmp.getKilled().size());
		}

		changeCurrentPlayer ();
	}
//	-------------------------------------------------------------------------*/
	public boolean canRedoMove () {
		return (myBufferCurrentIndex < myGameBuffer.size());
	}
//  -------------------------------------------------------------------------*/	
	public void redoMove () {
		if (!canRedoMove()) {
			throw new GoException ("Can not redoMove");
		}
		IMoveRecord tmp  = myGameBuffer.get(myBufferCurrentIndex++);
		
		if (!tmp.isPass()) {
			myBoard.setStone(tmp.getStone().getVertical(),tmp.getStone().getHorizontal(), tmp.getStone().getColor());
			
			for (IStone stone : tmp.getKilled()) {
				myBoard.removeStone(stone);
			}
			incScores (tmp.getStone().getColor(), tmp.getKilled().size());
		}
		
		changeCurrentPlayer();
	}
//   -------------------------------------------------------------------------
	public boolean modified () {
		return myModified && !myGameBuffer.isEmpty();
	}
//	-------------------------------------------------------------------------	
	public void justSaved () {
		myModified = false;
	}
//	-------------------------------------------------------------------------
	public void clear () {
		for (int vertical = 0; vertical < myBoard.getSize(); ++vertical) {
			for (int horizontal = 0; horizontal < myBoard.getSize(); ++horizontal) {
				IStone tmp = myBoard.getStone(vertical, horizontal);
				if (tmp != null) {
					myBoard.removeStone(tmp);
				}
			}
		}
		myWhitesScores = 0;
		myBlacksScores = 0;
		myBufferCurrentIndex = 0;
		myGameBuffer.clear();
		myCurrentPlayer = GoColor.BLACK;
		justSaved ();
	}
//	-------------------------------------------------------------------------
	public boolean isGameOver () {
		if (myBufferCurrentIndex >= 3) {
			if (myGameBuffer.get(myBufferCurrentIndex - 1).isPass() &&
				myGameBuffer.get(myBufferCurrentIndex - 2).isPass()) {
				return true;
			}
		}
		return false;
	}	
//	-------------------------------------------------------------------------	
	public List<IMoveRecord> getGameBuffer () {
		return Collections.unmodifiableList(myGameBuffer);
	}
//	-------------------------------------------------------------------------	
	public int getBufferCurrentIndex () {
		return myBufferCurrentIndex;
	}
//	-------------------------------------------------------------------------	
	public void setBufferCurrentIndex (int index) {
		myBufferCurrentIndex = index;
	}
//	-------------------------------------------------------------------------	
	private void incScores (GoColor color, int scores) {
		if (color == GoColor.WHITE) {
			myWhitesScores += scores;
		} else {
			myBlacksScores += scores;
		}
	}
	
//	-------------------------------------------------------------------------	
	public long getScores (GoColor color) {
		if (color == GoColor.WHITE) {
			return myWhitesScores;
		} else {
			return myBlacksScores;
		}
	}
}
