package org.amse.byBedrosova.go.view;

import java.awt.*;

import javax.swing.ImageIcon;
import javax.swing.JPanel;

import java.util.HashSet;
import java.util.Set;

import org.amse.byBedrosova.go.exception.GoException;
import org.amse.byBedrosova.go.model.*;

@SuppressWarnings("serial")

public class GoBoardPanel extends JPanel {	
	
	private int myElementSize = 1,
				myStoneRadius;
	
	private IGame myGame;
	
	private boolean myShowCursor;
	private boolean myShowNumbers;
	private boolean myLightUpSurrounded;
	
	public GoBoardPanel (IGame game) {
		super ();
		
		myGame = game;
		myShowCursor = true;
		myShowNumbers = false;
		myLightUpSurrounded = true;
	}
	
	public BoardPoint getCurrentBoardPoint (Point p) {
		Point [] vertex = new Point [4];
		
		int xNumber = p.x / myElementSize;
		int yNumber = p.y / myElementSize;
		vertex [0] = new Point (xNumber * myElementSize, yNumber * myElementSize);
		vertex [1] = new Point (xNumber * myElementSize, (yNumber + 1) * myElementSize);
		vertex [2] = new Point ((xNumber + 1) * myElementSize, yNumber * myElementSize);
		vertex [3] = new Point ((xNumber + 1) * myElementSize, (yNumber + 1) * myElementSize);

		for (int i = 0; i < 4; ++i) {
			if (Math.hypot(vertex [i].x - p.x, vertex [i].y - p.y) < myStoneRadius) {
				return new BoardPoint(myGame.getBoard(), vertex [i].x / myElementSize - 1, vertex [i].y / myElementSize - 1);
			}
		}

		return new BoardPoint();
	}	
	
	public void showCursor (boolean show) {
		myShowCursor = show;
	}
	
	public void showNumbers (boolean show) {
		myShowNumbers = show;
	}
	
	public boolean areNumbersShown () {
		return myShowNumbers;
	}
	
	public void lightUpSurrounded (boolean lightUp) {
		myLightUpSurrounded = lightUp;
	}
	
	public boolean areSurroundedLitUp () {
		return myLightUpSurrounded;
	}	                                          
	
	private void drawBoard (Graphics g) {
		myElementSize = getWidth () / (myGame.getBoard ().getSize() + 1);
		myStoneRadius = myElementSize / 2;
		
		for (int i = 1; i <= myGame.getBoard ().getSize(); ++i) {
			g.setColor(Color.BLUE);
			g.drawLine (i * myElementSize, 0, 
						i * myElementSize, getHeight ());  			
			g.drawLine (0, i * myElementSize, 
						getWidth (), i * myElementSize);
			if (myShowNumbers) {
				g.setColor(Color.BLACK);
				String tmp = new Integer (i).toString();
				g.setFont(new Font ("Arial", Font.BOLD | Font.ITALIC, 9));
				g.drawString(tmp, i * myElementSize - tmp.length() * 5, 10);
				g.drawString(tmp, 2, i * myElementSize);
			}
		}
		
		if (myGame.getBoard().getSize() == 19) {
			g.setColor(Color.BLUE);
			for (int i = 1; i <= 5; ++i) {
				for (int j = 1; j <= 5; ++j) {
					if ((i % 2 != 0) && (j % 2 != 0)) {
						g.fillOval(3 * i * myElementSize - 3 + myElementSize, 3 * j * myElementSize - 3 + myElementSize, 7, 7);
					}
				}
			}
		}		
	}
	
	private void drawStones (Graphics g) {
		Set<IStone> set = new HashSet<IStone> ();

		if (!myGame.isGameOver()) {		
	 		Point p = getMousePosition();			
	
			if (p != null && myShowCursor) {				
				BoardPoint tmpBoardPoint =  getCurrentBoardPoint(p);
				
				if (tmpBoardPoint.isValid()) {				
					int vertical = tmpBoardPoint.getHorizontal();
					int horizontal = tmpBoardPoint.getVertical();			
				
					if (myGame.canDoMove(vertical, horizontal) ) {
						drawStone (g, vertical, horizontal, 
								   myGame.getCurrentPlayer(), false, true);								
						collectSurrounded (vertical, horizontal, set);
					} else if (myGame.getBoard().getStone(vertical, horizontal) == null) {
						drawStone (g, vertical, horizontal,
								myGame.getCurrentPlayer(), true, true);				
					}
				}
			}
		}
		for (int vertical = 0; vertical < myGame.getBoard().getSize(); ++vertical) {
			for (int horizontal = 0; horizontal < myGame.getBoard ().getSize(); ++horizontal) {
				if (myGame.getBoard().getStone(vertical, horizontal) != null) {
					if (myLightUpSurrounded && set.contains(myGame.getBoard().getStone(vertical, horizontal))) {
							drawStone (g, vertical, horizontal, GoColor.RED, false, false);	
					} else {
						drawStone (g, vertical, horizontal, myGame.getBoard().getStone(vertical, horizontal).getColor(), false, false);						
					}
				}
			}
		}		
	}
	
	public void paint (Graphics g) {
		super.paint(g);			
		drawBoard(g);
		drawStones(g);
	}
	
	public void drawStone (Graphics g, int vertical, int horizontal, GoColor color, boolean crossed, boolean transparent) {		
		ImageIcon image;
		if (transparent) {
			if (crossed) {
				image = color.getCrossedIcon();				
			}
			else {
				image = color.getTransparentIcon();
			}
		} else {
			if (crossed) {
				throw new GoException ("Crossed must be transparent");
			}
			image = color.getIcon();				
		}
		g.drawImage(image.getImage(), (horizontal + 1) * myElementSize - myElementSize / 2, 
				   (vertical + 1) * myElementSize - myElementSize / 2,
				   (horizontal + 1) * myElementSize + myElementSize / 2,
				   (vertical + 1) * myElementSize + myElementSize / 2,
				   0, 0, image.getIconWidth(), image.getIconHeight(),
					this);							
	}
	
	private IGroup getGroup (int vertical, int horizontal) {
		if (myGame.getBoard().getStone(vertical, horizontal) != null) {
			if (myGame.getBoard().getStone(vertical, horizontal).getColor() != myGame.getCurrentPlayer()) {
				return myGame.getBoard().collectGroup(myGame.getBoard().getStone(vertical, horizontal));
			}
		}
		return null;
	}
	
	private void collectSurrounded (int vertical, int horizontal, Set<IStone> set) {
		IGroup group;
		
		if ((group = getGroup (vertical - 1, horizontal)) != null) {
			if (myGame.getBoard().isGroupSurrounded(group, vertical, horizontal)) {
				set.addAll(group.stones());
			}
		}
		if ((group = getGroup (vertical, horizontal + 1)) != null) {
			if (myGame.getBoard().isGroupSurrounded(group, vertical, horizontal)) {
				set.addAll(group.stones());
			}
		}
		if ((group = getGroup (vertical + 1, horizontal)) != null) {
			if (myGame.getBoard().isGroupSurrounded(group, vertical, horizontal)) {
				set.addAll(group.stones());
			}
		}
		if ((group = getGroup (vertical, horizontal - 1)) != null) {
			if (myGame.getBoard().isGroupSurrounded(group, vertical, horizontal)) {
				set.addAll(group.stones());
			}
		}	
	}
	
	public void drawMove () {
		Point p = getMousePosition();
		
		if (p == null) {
			return;
		}
		
		BoardPoint tmpBoardPoint = getCurrentBoardPoint(p);
		
		if (tmpBoardPoint.isValid()) {			
			if (myGame.canDoMove (tmpBoardPoint.getHorizontal(), tmpBoardPoint.getVertical())) {
				myGame.doMove (tmpBoardPoint.getHorizontal(), tmpBoardPoint.getVertical());
			}
		}
	}
	
	public void setGame (IGame game) {
		myGame = game;
	}
}	
