package org.amse.byBedrosova.go.view;

import javax.swing.border.LineBorder;
import javax.swing.*;
import javax.swing.event.MouseInputAdapter;
import javax.swing.filechooser.FileFilter;

import java.awt.*;
import java.awt.event.*;

import java.util.List;
import java.util.ArrayList;

import java.io.*;

import org.amse.byBedrosova.go.exception.GoException;
import org.amse.byBedrosova.go.io.GoReader;
import org.amse.byBedrosova.go.io.GoWriter;
import org.amse.byBedrosova.go.model.GoColor;
import org.amse.byBedrosova.go.model.IGame;
import org.amse.byBedrosova.go.model.IMoveRecord;
import org.amse.byBedrosova.go.model.IStone;
import org.amse.byBedrosova.go.model.impl.ModelFactory;

@SuppressWarnings("serial")

public class View extends JFrame {
	
	public static final String DLG_CURRENT_DIRECTORY = ".";
	private static final String HOME_DIRECTORY = System.getProperty("user.home"); 
	
	private IGame myGame;
	
	private GoBoardPanel myPanel;
	private GoToolBar myToolBar;
	private JMenuBar myMenuBar;
	
	private JLabel myCurrentPlayerLabel;
	private JLabel myPreviousMoveLabel;
	private JLabel myBlackScoresLabel;
	private JLabel myWhiteScoresLabel;
	
	private MouseInputAdapter myMouseInputAdapter;
	private List<Action> myActions = new ArrayList<Action>();
	
	public View () {
		super ("GO");		
		{
			ImageIcon icon = loadIcon("/icons/new.gif");
			setIconImage(icon.getImage());
		}
		
		myGame = new ModelFactory ().newGame(19);
		
		File source = new File (HOME_DIRECTORY + File.separator + "goSave");
		if (source.exists()) {
			try {
				(new GoReader (myGame)).loadGame(source.getPath());
				myGame.justSaved();
			} catch (GoException e) {
			}
		}	
		myMenuBar = new JMenuBar();
		JMenu fileMenu = myMenuBar.add(new JMenu("File"));
		JMenu moveMenu = myMenuBar.add(new JMenu("Move"));
		JMenu viewMenu = myMenuBar.add(new JMenu("View"));		
		
		setJMenuBar(myMenuBar);
		
		myToolBar = new GoToolBar (); 

		myPanel = new GoBoardPanel (myGame);
		myMouseInputAdapter = new GoMouseListener ();
		myPanel.addMouseMotionListener(myMouseInputAdapter);
		myPanel.addMouseListener(myMouseInputAdapter);		
		
		this.addWindowListener(new GoWindowListener ());		
		
		addAction(new GoNewGameAction (this), fileMenu, myToolBar);
		addAction(new GoLoadGameAction (this), fileMenu, myToolBar);
		addAction(new GoSaveGameAction (this), fileMenu, myToolBar);		

		myToolBar.addMySeparator();
		
		addAction(new GoUndoAction (this), moveMenu, myToolBar);
		addAction(new GoRedoAction (this), moveMenu, myToolBar);
		addAction(new GoPassAction(this), moveMenu, myToolBar);

		myToolBar.addMySeparator();
		fileMenu.addSeparator();

		addAction(new GoExitAction (this), fileMenu, myToolBar);
		
		myToolBar.addMySeparator();
		
		JMenu aboutMenu = myMenuBar.add( new JMenu("Help") ); 
		aboutMenu.add(new AbstractAction ("About Go...") {
			public void actionPerformed(ActionEvent arg0) {
					getPanel().showCursor(false);
					JTextPane tp = new JTextPane();
					tp.setEditable(false);
					tp.setText(new GoReader (View.this).loadAbout("readme"));
					JOptionPane.showMessageDialog(View.this, tp, "About Go", JOptionPane.INFORMATION_MESSAGE);			
					getPanel().showCursor(true);
			}
		});
		
		myCurrentPlayerLabel = new JLabel ();
		myCurrentPlayerLabel.setHorizontalTextPosition(SwingConstants.LEFT);
		myCurrentPlayerLabel.setPreferredSize(new Dimension (150, 18));
		myToolBar.add(myCurrentPlayerLabel);

		myPreviousMoveLabel = new JLabel ();
		myPreviousMoveLabel.setHorizontalTextPosition(SwingConstants.LEFT);		
		myPreviousMoveLabel.setPreferredSize(new Dimension (150, 18));
		myToolBar.add(myPreviousMoveLabel);

		myBlackScoresLabel = new JLabel ();
		myBlackScoresLabel.setPreferredSize(new Dimension (150, 13));
		myToolBar.add(myBlackScoresLabel);
		myWhiteScoresLabel = new JLabel ();
		myWhiteScoresLabel.setPreferredSize(new Dimension (150, 13));
		myToolBar.add(myWhiteScoresLabel);

		getContentPane ().setLayout(new BorderLayout ());		
		
		myPanel.setPreferredSize(new Dimension (400, 400));
		myPanel.setBackground(Color.LIGHT_GRAY);
		myPanel.setBorder(new LineBorder (Color.BLUE, 2));
		
		myToolBar.setPreferredSize(new Dimension (200, 400));
		myToolBar.setBackground(Color.LIGHT_GRAY);
		myToolBar.setBorder(new LineBorder (Color.BLUE, 2));

		getContentPane ().add(myPanel, BorderLayout.WEST);
		getContentPane ().add(myToolBar, BorderLayout.EAST);

		source = new File (HOME_DIRECTORY + File.separator + "goSettings");
		if (source.exists()) {
			try {
				(new GoReader (this)).loadSettings(source.getPath());
			} catch (GoException e) {
				Dimension scrSize = Toolkit.getDefaultToolkit().getScreenSize();
				setLocation(scrSize.width / 2 - 300, scrSize.height / 2 - 300);
				myToolBar.setVisible(true);
				myPanel.lightUpSurrounded(true);
				myPanel.showNumbers(false);
			}
		} else {
			myToolBar.setVisible(true);
			myPanel.lightUpSurrounded(true);
			myPanel.showNumbers(false);
		}
		inicializeViewMenu(viewMenu);
		setResizable(false);
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		pack ();
		updateAll ();
		setVisible(true);	
	}
	
	private void addAction (Action action, JMenu menu, GoToolBar toolBar) {
		toolBar.addButton (action);
		menu.add(action).setIcon(null);
		myActions.add(action);		
	}
	
	public void inicializeViewMenu (JMenu viewMenu) {
		JCheckBoxMenuItem a;
		
		a = new JCheckBoxMenuItem (new AbstractAction("Show tool bar") {
			public void actionPerformed(ActionEvent arg0) {
				myToolBar.setVisible(!myToolBar.isVisible());
				pack();
			}			
		}); 
		a.setState(myToolBar.isVisible());
		viewMenu.add(a);		

		a = new JCheckBoxMenuItem (new AbstractAction ("Show board numbers") {
			public void actionPerformed(ActionEvent arg0) {
				myPanel.showNumbers(!myPanel.areNumbersShown());
				updateAll();
			}			
		}); 
		a.setState(myPanel.areNumbersShown());
		viewMenu.add(a);

		a = new JCheckBoxMenuItem (new AbstractAction ("Light up surrounded stones") {
			public void actionPerformed(ActionEvent arg0) {
				myPanel.lightUpSurrounded(!myPanel.areSurroundedLitUp());
				updateAll();
			}			
		}); 
		a.setState(myPanel.areSurroundedLitUp());
		viewMenu.add(a);		
	}
	
	public static ImageIcon loadIcon (String path) {
		java.net.URL url = View.class.getResource(path);
		if (url == null) {
			throw new GoException ("No icon " + path);
		}
		return new ImageIcon (url);
	}
	
	public void updateAll() {
		for (Action action : myActions) {
			action.setEnabled(action.isEnabled());
		}
		if (!myGame.isGameOver()) {
			myCurrentPlayerLabel.setIcon(myGame.getCurrentPlayer().getIcon());
			myCurrentPlayerLabel.setText("Current palyer : ");
		} else {
			myCurrentPlayerLabel.setIcon(null);
			myCurrentPlayerLabel.setText("Game over");
		}
		if (myGame.getBufferCurrentIndex() != 0) {
			IMoveRecord tmpMove = myGame.getGameBuffer().get(myGame.getBufferCurrentIndex() - 1);
			if (tmpMove.isPass()) {
				myPreviousMoveLabel.setText("Previous move : pass");				
			} else {
				IStone tmpStone = tmpMove.getStone();
				myPreviousMoveLabel.setText("Previous move : " + (tmpStone.getVertical() + 1) + ", " + (tmpStone.getHorizontal() + 1));
			}
		} else {
			myPreviousMoveLabel.setIcon(null);
			myPreviousMoveLabel.setText(null);			
		}
		myBlackScoresLabel.setText("Black scores : " + myGame.getScores(GoColor.BLACK));
		myWhiteScoresLabel.setText("White scores : " + myGame.getScores(GoColor.WHITE));
		repaint();
	}
	
	public void exitDialog () {
/*		if (myGame.modified()) {
			switch (JOptionPane.showConfirmDialog (View.this, "Do you want to save current game?",
					"Exit",
					JOptionPane.YES_NO_CANCEL_OPTION)) {
				case JOptionPane.YES_OPTION : 
					if (saveDialog ()) {
						System.exit(0);
					}
					break;
				
				case JOptionPane.NO_OPTION :
					System.exit(0);
					break;
				
				default:
					break;
			}
			return;
		} else {*/
			//if (JOptionPane.showConfirmDialog (View.this, "Do you want to close program?",
				//	"Exit",
					//JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION) {
				try {
					new GoWriter (myGame).saveGame(HOME_DIRECTORY + File.separator + "goSave");
					new GoWriter (this).saveViewSettings(HOME_DIRECTORY + File.separator + "goSettings");
				} catch (GoException e) {
					//e.printStackTrace();
				}
				System.exit(0);
			//}				
		//}
	}
	
	public boolean saveDialog () {
		JFileChooser fileChooser = new JFileChooser (DLG_CURRENT_DIRECTORY);

		FileFilter filter = new GoFileFilter ();
		
		fileChooser.setFileFilter(filter); 

		fileChooser.setDialogTitle ("Save game");
		//fileChooser.setMultiSelectionEnabled(false);
		fileChooser.setFileSelectionMode (JFileChooser.FILES_ONLY);
		
		int tmp = fileChooser.showSaveDialog (this);
		
		if (tmp == JFileChooser.APPROVE_OPTION) {
			try {
				if (fileChooser.getSelectedFile ().exists()) {
					switch (JOptionPane.showConfirmDialog(View.this, "File already exists!\nDo you want to replace it?",
													      "Save game", JOptionPane.YES_NO_CANCEL_OPTION)) {
					case JOptionPane.YES_OPTION : 
						break;
					default : 
						return false;
					}
				}
				
				if (fileChooser.getSelectedFile ().getName().contains(".")) {					
					(new GoWriter (myGame)).saveGame(fileChooser.getSelectedFile ().getPath());
				} else {
					(new GoWriter (myGame)).saveGame(fileChooser.getSelectedFile ().getPath() + ".go");
				}
			} catch (GoException ex) {
				JOptionPane.showMessageDialog (View.this, "Couldn't save game to file!", "Save game", JOptionPane.ERROR_MESSAGE);
				return false;						
			}
			myGame.justSaved();
			return true;
		}		
		return false;
	}
	
	public void gameOverDialog () {
		JOptionPane.showMessageDialog (this, "Both players passed - game over!", "Game over", JOptionPane.INFORMATION_MESSAGE);
	}
	
	public void loadGameDialog() {
		JFileChooser fileChooser = new JFileChooser (View.DLG_CURRENT_DIRECTORY);
		
		FileFilter filter = new GoFileFilter ();
		
		fileChooser.setFileFilter(filter); 
		
		fileChooser.setDialogTitle ("Load game");
		fileChooser.setFileSelectionMode (JFileChooser.FILES_ONLY);
		
		int tmp = fileChooser.showOpenDialog (this);
		
		if (tmp == JFileChooser.APPROVE_OPTION) {
			try {
				myGame.clear();
				(new GoReader (myGame)).loadGame(fileChooser.getSelectedFile ().getPath());
			} catch (GoException ex) {
				JOptionPane.showMessageDialog (this, "Couldn't load game from file!", "Load game", JOptionPane.ERROR_MESSAGE);
			}
			myGame.justSaved();
		}
	}
	
	
	public IGame getGame () {
		return myGame;
	}
	
	public GoBoardPanel getPanel() {
		return myPanel;
	}
	
	public GoToolBar getToolBar () {
		return myToolBar;
	}
	
	private class GoMouseListener extends MouseInputAdapter {
		public void mouseClicked (MouseEvent e) {
			myPanel.drawMove();
			View.this.updateAll();
		}

		public void mouseMoved (MouseEvent e) {
			View.this.repaint ();
		}	
		
		public void mouseExited (MouseEvent e) {
			View.this.repaint();			
		}
	}	
	
	private class GoWindowListener extends WindowAdapter {
		public void windowClosing (WindowEvent e){
			getPanel().showCursor(false);			
			updateAll();
			exitDialog ();
			getPanel().showCursor(true);
			updateAll();
		}
	}
}
