package org.amse.byBedrosova.tests;

import junit.framework.*;

import org.amse.byBedrosova.go.exception.GoException;
import org.amse.byBedrosova.go.model.*;
import org.amse.byBedrosova.go.model.impl.ModelFactory;

public class ModelTests extends TestCase {

	public void test1 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		for (int i = 0; i < game.getBoard().getSize(); ++i) {
			for (int j = 0; j < game.getBoard().getSize(); ++j) {
				game.getBoard().setStone(i, j, GoColor.BLACK);
			}
		}		
		
		for (int i = 0; i < game.getBoard().getSize(); ++i) {
			for (int j = 0; j < game.getBoard().getSize(); ++j) {
				assertNotNull ("setStone/getStone test failed", 
							  game.getBoard().getStone(i, j));
			}
		}		
	}

	public void test2 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		for (int i = 0; i < game.getBoard().getSize(); ++i) {
			for (int j = 0; j < game.getBoard().getSize(); ++j) {				
				 assertTrue ("setStone test for blacks failed", 
						 	  game.canDoMove(i, j));
			}
		}
		
		game.changeCurrentPlayer();

		for (int i = 0; i < game.getBoard().getSize(); ++i) {
			for (int j = 0; j < game.getBoard().getSize(); ++j) {
				 assertTrue ("setStone test for whites failed", 
						 	  game.canDoMove(i, j));
			}
		}
	}

	public void test3 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		for (int i = 0; i < game.getBoard().getSize(); ++i) {
			for (int j = 0; j < game.getBoard().getSize(); ++j) {
				game.getBoard().setStone(i, j, GoColor.BLACK);
			}
		}
		
		for (int i = 0; i < game.getBoard().getSize(); ++i) {
			for (int j = 0; j < game.getBoard().getSize(); ++j) {
				game.getBoard().removeStone(game.getBoard().getStone(i, j));
			}
		}		

		for (int i = 0; i < game.getBoard().getSize(); ++i) {
			for (int j = 0; j < game.getBoard().getSize(); ++j) {
				assertTrue ("removeStone/canSetStone test for blacks failed",
							 game.canDoMove(i, j));
			}
		}

		game.changeCurrentPlayer();
		
		for (int i = 0; i < game.getBoard().getSize(); ++i) {
			for (int j = 0; j < game.getBoard().getSize(); ++j) {
				assertTrue ("removeStone/canSetStone test for blacks failed",
							 game.canDoMove(i, j));
			}
		}
	}

	public void test4 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		for (int i = 0; i < game.getBoard().getSize(); ++i) {
			for (int j = 0; j < game.getBoard().getSize(); ++j) {
				game.getBoard().setStone(i, j, GoColor.BLACK);
			}
		}
		
		for (int i = 0; i < game.getBoard().getSize(); ++i) {
			for (int j = 0; j < game.getBoard().getSize(); ++j) {
				game.getBoard().removeStone(game.getBoard().getStone(i, j));
			}
		}		

		for (int i = 0; i < game.getBoard().getSize(); ++i) {
			for (int j = 0; j < game.getBoard().getSize(); ++j) {
				assertNull ("removeStone test failed", 
						   	 game.getBoard().getStone(i, j));
			}
		}
	}

	public void test5 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		int j;
		int i = j = game.getBoard().getSize() / 2; 
		
		game.getBoard().setStone(i, j, GoColor.WHITE);
		
		assertFalse("setStone test failed",
				game.canDoMove(i, j));
		
		game.changeCurrentPlayer();

		assertFalse("setStone test failed",
					game.canDoMove(i, j));
		
		game.getBoard().removeStone(game.getBoard().getStone(i, j));
		
		assertTrue ("setStone/removeStone test failed",
					game.canDoMove(i, j));

		game.changeCurrentPlayer();
		
		assertTrue("setStone test failed",
					game.canDoMove(i, j));
		
	}

	public void test6 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		game.getBoard().setStone(1, 3, GoColor.WHITE);

		game.getBoard().setStone(2, 2, GoColor.WHITE);
		game.getBoard().setStone(2, 3, GoColor.BLACK);
		game.getBoard().setStone(2, 4, GoColor.WHITE);

		game.getBoard().setStone(3, 2, GoColor.WHITE);
		game.getBoard().setStone(3, 4, GoColor.WHITE);
		game.getBoard().setStone(3, 5, GoColor.WHITE);

		game.getBoard().setStone(4, 2, GoColor.BLACK);
		game.getBoard().setStone(4, 3, GoColor.WHITE);
		game.getBoard().setStone(4, 4, GoColor.BLACK);
		game.getBoard().setStone(4, 6, GoColor.WHITE);

		game.getBoard().setStone(5, 2, GoColor.BLACK);
		game.getBoard().setStone(5, 3, GoColor.WHITE);
		game.getBoard().setStone(5, 4, GoColor.BLACK);
		game.getBoard().setStone(5, 5, GoColor.BLACK);
		game.getBoard().setStone(5, 6, GoColor.WHITE);

		game.getBoard().setStone(6, 2, GoColor.BLACK);
		game.getBoard().setStone(6, 3, GoColor.BLACK);
		game.getBoard().setStone(6, 4, GoColor.WHITE);
		game.getBoard().setStone(6, 5, GoColor.WHITE);
		
		assertTrue ("canSetStone for blacks failed",
					 game.canDoMove (3, 3));

		assertFalse ("canSetStone for blacks failed",
			 	  game.canDoMove (4, 5));

		game.changeCurrentPlayer();

		assertTrue ("canSetStone for whites failed",
					 game.canDoMove (3, 3) && 
					 game.canDoMove (4, 5));
	}

	public void test7 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		game.getBoard().setStone(1, 3, GoColor.WHITE);
		game.getBoard().setStone(2, 2, GoColor.WHITE);
		game.getBoard().setStone(2, 3, GoColor.BLACK);
		game.getBoard().setStone(2, 4, GoColor.WHITE);
		game.getBoard().setStone(3, 2, GoColor.WHITE);
		game.getBoard().setStone(3, 4, GoColor.WHITE);
		game.getBoard().setStone(3, 5, GoColor.WHITE);
		game.getBoard().setStone(4, 2, GoColor.BLACK);
		game.getBoard().setStone(4, 3, GoColor.WHITE);
		game.getBoard().setStone(4, 4, GoColor.BLACK);
		game.getBoard().setStone(4, 6, GoColor.WHITE);
		game.getBoard().setStone(5, 2, GoColor.BLACK);
		game.getBoard().setStone(5, 3, GoColor.WHITE);
		game.getBoard().setStone(5, 4, GoColor.BLACK);
		game.getBoard().setStone(5, 5, GoColor.BLACK);
		game.getBoard().setStone(5, 6, GoColor.WHITE);
		game.getBoard().setStone(6, 2, GoColor.BLACK);
		game.getBoard().setStone(6, 3, GoColor.BLACK);
		game.getBoard().setStone(6, 4, GoColor.WHITE);
		game.getBoard().setStone(6, 5, GoColor.WHITE);

		game.changeCurrentPlayer();

		for (int i = 0; i < game.getBoard().getSize(); ++i) {
			for (int j = 0; j < game.getBoard().getSize(); ++j) {
				if (!game.canDoMove(i, j)) {
					if (game.getBoard().getStone(i, j) == null) {
						fail ("canSetStone test failed");						
					}
				}
			}
		}		
	}

	public void test8 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		game.getBoard().setStone(1, 0, GoColor.WHITE);
		game.getBoard().setStone(1, 1, GoColor.WHITE);
		game.getBoard().setStone(0, 1, GoColor.WHITE);
		
		game.getBoard().setStone(0, game.getBoard().getSize() - 2, GoColor.BLACK);
		game.getBoard().setStone(1, game.getBoard().getSize() - 2, GoColor.BLACK);
		game.getBoard().setStone(1, game.getBoard().getSize() - 1, GoColor.BLACK);
		
		game.getBoard().setStone(game.getBoard().getSize() - 2, 0, GoColor.WHITE);
		game.getBoard().setStone(game.getBoard().getSize() - 2, 1, GoColor.WHITE);
		game.getBoard().setStone(game.getBoard().getSize() - 1, 1, GoColor.WHITE);		

		game.getBoard().setStone(game.getBoard().getSize() - 1, game.getBoard().getSize() - 2, GoColor.BLACK);
		game.getBoard().setStone(game.getBoard().getSize() - 2, game.getBoard().getSize() - 2, GoColor.BLACK);
		game.getBoard().setStone(game.getBoard().getSize() - 2, game.getBoard().getSize() - 1, GoColor.BLACK);		
		
		assertTrue ("canSetStone test failed",
					 game.canDoMove(0, game.getBoard().getSize() - 1) &&
					 game.canDoMove(game.getBoard().getSize() - 1, 
					  				   game.getBoard().getSize() - 1));
		
		assertFalse ("canSetStone test failed",
					  game.canDoMove(0, 0)                   ||
					  game.canDoMove(game.getBoard().getSize() - 1, 0));

		game.changeCurrentPlayer();

		assertTrue ("canSetStone test failed",
				 	 game.canDoMove(0, 0) &&
				 	 game.canDoMove(game.getBoard().getSize() - 1, 0));

		assertFalse ("canSetStone test failed",
				  game.canDoMove(0, game.getBoard().getSize() - 1) ||
				  game.canDoMove(game.getBoard().getSize() - 1, 
                                   game.getBoard().getSize() - 1));
	}

	public void test9 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		game.getBoard().setStone(1, 0, GoColor.WHITE);
		game.getBoard().setStone(1, 1, GoColor.WHITE);
		game.getBoard().setStone(0, 1, GoColor.WHITE);
		
		game.getBoard().setStone(2, 0, GoColor.WHITE);
		game.getBoard().setStone(2, 1, GoColor.WHITE);
		game.getBoard().setStone(0, 2, GoColor.WHITE);
		game.getBoard().setStone(1, 2, GoColor.WHITE);

		game.getBoard().setStone(3, 0, GoColor.WHITE);
		game.getBoard().setStone(3, 1, GoColor.WHITE);
		game.getBoard().setStone(3, 2, GoColor.WHITE);
		game.getBoard().setStone(3, 3, GoColor.WHITE);
		game.getBoard().setStone(0, 3, GoColor.WHITE);
		game.getBoard().setStone(1, 3, GoColor.WHITE);
		game.getBoard().setStone(2, 3, GoColor.WHITE);		
	
		assertFalse ("canSetStone test failed",
					 game.canDoMove(0, 0) &&
					 game.canDoMove(2, 2));

		game.changeCurrentPlayer();
		
		assertTrue ("canSetStone test failed",
				 	  game.canDoMove(0, 0) ||
				 	  game.canDoMove(2, 2));
	}
	

	public void test10 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		game.getBoard().setStone(1, 0, GoColor.WHITE);
		game.getBoard().setStone(1, 1, GoColor.WHITE);
		game.getBoard().setStone(0, 1, GoColor.WHITE);
		
		game.getBoard().setStone(2, 0, GoColor.BLACK);
		game.getBoard().setStone(2, 1, GoColor.BLACK);
		game.getBoard().setStone(0, 2, GoColor.BLACK);
		game.getBoard().setStone(1, 2, GoColor.BLACK);

		game.getBoard().setStone(3, 0, GoColor.WHITE);
		game.getBoard().setStone(3, 1, GoColor.WHITE);
		game.getBoard().setStone(3, 2, GoColor.WHITE);
		game.getBoard().setStone(3, 3, GoColor.WHITE);
		game.getBoard().setStone(0, 3, GoColor.WHITE);
		game.getBoard().setStone(1, 3, GoColor.WHITE);
		game.getBoard().setStone(2, 3, GoColor.WHITE);		
	
		assertFalse ("canSetStone test failed",
					 game.canDoMove(0, 0) &&
					 game.canDoMove(2, 2));

		game.changeCurrentPlayer();

		assertTrue ("canSetStone test failed",
			 	  	  game.canDoMove(0, 0) ||
			 	  	  game.canDoMove(2, 2));
	}

	public void test11 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		game.getBoard().setStone(0, 0, GoColor.WHITE);
		game.getBoard().setStone(0, 1, GoColor.WHITE);
		game.getBoard().setStone(0, 2, GoColor.WHITE);
		game.getBoard().setStone(0, 3, GoColor.WHITE);
		game.getBoard().setStone(0, 4, GoColor.WHITE);

		game.getBoard().setStone(1, 0, GoColor.WHITE);
		game.getBoard().setStone(1, 1, GoColor.BLACK);
		game.getBoard().setStone(1, 3, GoColor.BLACK);
		game.getBoard().setStone(1, 4, GoColor.WHITE);

		game.getBoard().setStone(2, 0, GoColor.WHITE);
		game.getBoard().setStone(2, 1, GoColor.WHITE);
		game.getBoard().setStone(2, 2, GoColor.WHITE);
		game.getBoard().setStone(2, 3, GoColor.WHITE);
		game.getBoard().setStone(2, 4, GoColor.WHITE);
		
		assertFalse("canSetStoneTest failed",
					 game.canDoMove(1, 2));
		game.changeCurrentPlayer();

		assertTrue("canSetStone test failed",
					game.canDoMove(1, 2));
	}

	public void test12 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		game.getBoard().setStone(1, 1, GoColor.BLACK);
		game.getBoard().setStone(1, 2, GoColor.BLACK);
		game.getBoard().setStone(1, 3, GoColor.BLACK);
		game.getBoard().setStone(1, 4, GoColor.BLACK);

		game.getBoard().setStone(2, 1, GoColor.BLACK);
		game.getBoard().setStone(2, 2, GoColor.WHITE);
		game.getBoard().setStone(2, 3, GoColor.WHITE);
		game.getBoard().setStone(2, 4, GoColor.BLACK);

		game.getBoard().setStone(3, 1, GoColor.BLACK);
		game.getBoard().setStone(3, 2, GoColor.WHITE);
		game.getBoard().setStone(3, 3, GoColor.WHITE);
		game.getBoard().setStone(3, 4, GoColor.BLACK);

		game.getBoard().setStone(4, 1, GoColor.BLACK);
		game.getBoard().setStone(4, 2, GoColor.BLACK);
		game.getBoard().setStone(4, 3, GoColor.BLACK);
		game.getBoard().setStone(4, 4, GoColor.BLACK);		

		IGroup g1 = game.getBoard().collectGroup(game.getBoard().getStone(2, 2));
		IGroup g2 = game.getBoard().collectGroup(game.getBoard().getStone(2, 3));
		IGroup g3 = game.getBoard().collectGroup(game.getBoard().getStone(3, 2));		
		IGroup g4 = game.getBoard().collectGroup(game.getBoard().getStone(3, 3));
		
		assertTrue ("collectGroup/isGroupSurrounded test failed",
				     game.getBoard().isGroupSurrounded(g1) &&
				     game.getBoard().isGroupSurrounded(g2) &&
				     game.getBoard().isGroupSurrounded(g3) &&
				     game.getBoard().isGroupSurrounded(g4));		
	}

	public void test13 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		game.getBoard().setStone(game.getBoard().getSize() - 2, 0, GoColor.BLACK);
		game.getBoard().setStone(game.getBoard().getSize() - 2, 1, GoColor.BLACK);
		game.getBoard().setStone(game.getBoard().getSize() - 2, 2, GoColor.BLACK);
		game.getBoard().setStone(game.getBoard().getSize() - 2, 3, GoColor.BLACK);
		game.getBoard().setStone(game.getBoard().getSize() - 1, 0, GoColor.WHITE);
		game.getBoard().setStone(game.getBoard().getSize() - 1, 1, GoColor.WHITE);
		game.getBoard().setStone(game.getBoard().getSize() - 1, 2, GoColor.WHITE);
		game.getBoard().setStone(game.getBoard().getSize() - 1, 3, GoColor.BLACK);
	
		IGroup g1 = game.getBoard().collectGroup(game.getBoard().getStone(game.getBoard().getSize() - 1, 0));
		IGroup g2 = game.getBoard().collectGroup(game.getBoard().getStone(game.getBoard().getSize() - 1, 1));
		IGroup g3 = game.getBoard().collectGroup(game.getBoard().getStone(game.getBoard().getSize() - 1, 2));
		
		assertTrue ("collectGroup/isGroupSurrounded test failed",
					 game.getBoard().isGroupSurrounded(g1) &&
					 game.getBoard().isGroupSurrounded(g2) &&
					 game.getBoard().isGroupSurrounded(g3));
	}

	public void test14 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		game.getBoard().setStone(game.getBoard().getSize() - 2, 0, GoColor.BLACK);
		game.getBoard().setStone(game.getBoard().getSize() - 2, 1, GoColor.BLACK);
		game.getBoard().setStone(game.getBoard().getSize() - 2, 2, GoColor.BLACK);
		game.getBoard().setStone(game.getBoard().getSize() - 2, 3, GoColor.BLACK);
		game.getBoard().setStone(game.getBoard().getSize() - 1, 0, GoColor.WHITE);
		game.getBoard().setStone(game.getBoard().getSize() - 1, 1, GoColor.WHITE);
		game.getBoard().setStone(game.getBoard().getSize() - 1, 2, GoColor.WHITE);
		game.getBoard().setStone(game.getBoard().getSize() - 1, 3, GoColor.BLACK);
		
		IGroup g1 = game.getBoard().collectGroup(game.getBoard().getStone(game.getBoard().getSize() - 1, 0));

		if (game.getBoard().isGroupSurrounded(g1))
				game.getBoard().removeGroup(g1);

		assertTrue ("collectGroup/isGroupSurrounded/removeGroup test failed",
					 (game.getBoard().getStone(game.getBoard().getSize() - 1, 0) == null) &&
					 (game.getBoard().getStone(game.getBoard().getSize() - 1, 1) == null) &&
					 (game.getBoard().getStone(game.getBoard().getSize() - 1, 2) == null));

		assertFalse ("collectGroup/isGroupSurrounded/removeGroup test failed",
				 	 (game.getBoard().getStone(game.getBoard().getSize() - 2, 0) == null) ||
				 	 (game.getBoard().getStone(game.getBoard().getSize() - 2, 1) == null) ||
				 	 (game.getBoard().getStone(game.getBoard().getSize() - 2, 2) == null) ||
				 	 (game.getBoard().getStone(game.getBoard().getSize() - 2, 3) == null) ||
				 	 (game.getBoard().getStone(game.getBoard().getSize() - 1, 3) == null));
	}

	public void test15 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		game.getBoard().setStone(game.getBoard().getSize() - 2, 0, GoColor.BLACK);
		game.getBoard().setStone(game.getBoard().getSize() - 2, 1, GoColor.BLACK);
		game.getBoard().setStone(game.getBoard().getSize() - 2, 2, GoColor.BLACK);
		game.getBoard().setStone(game.getBoard().getSize() - 2, 3, GoColor.BLACK);
		game.getBoard().setStone(game.getBoard().getSize() - 1, 0, GoColor.WHITE);
		game.getBoard().setStone(game.getBoard().getSize() - 1, 1, GoColor.WHITE);
		game.getBoard().setStone(game.getBoard().getSize() - 1, 2, GoColor.WHITE);
		game.getBoard().setStone(game.getBoard().getSize() - 1, 3, GoColor.BLACK);
		
		IGroup g1 = game.getBoard().collectGroup(game.getBoard().getStone(game.getBoard().getSize() - 1, 1));

		if (game.getBoard().isGroupSurrounded(g1))
				game.getBoard().removeGroup(g1);

		assertTrue ("collectGroup/isGroupSurrounded/removeGroup test failed",
				 (game.getBoard().getStone(game.getBoard().getSize() - 1, 0) == null) &&
				 (game.getBoard().getStone(game.getBoard().getSize() - 1, 1) == null) &&
				 (game.getBoard().getStone(game.getBoard().getSize() - 1, 2) == null));

		assertFalse ("collectGroup/isGroupSurrounded/removeGroup test failed",
					 (game.getBoard().getStone(game.getBoard().getSize() - 2, 0) == null) ||
					 (game.getBoard().getStone(game.getBoard().getSize() - 2, 1) == null) ||
					 (game.getBoard().getStone(game.getBoard().getSize() - 2, 2) == null) ||
					 (game.getBoard().getStone(game.getBoard().getSize() - 2, 3) == null) ||
					 (game.getBoard().getStone(game.getBoard().getSize() - 1, 3) == null));
	}	

	public void test16 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		game.getBoard().setStone(game.getBoard().getSize() - 2, 0, GoColor.BLACK);
		game.getBoard().setStone(game.getBoard().getSize() - 2, 1, GoColor.BLACK);
		game.getBoard().setStone(game.getBoard().getSize() - 2, 2, GoColor.BLACK);
		game.getBoard().setStone(game.getBoard().getSize() - 2, 3, GoColor.BLACK);
		game.getBoard().setStone(game.getBoard().getSize() - 1, 0, GoColor.WHITE);
		game.getBoard().setStone(game.getBoard().getSize() - 1, 1, GoColor.WHITE);
		game.getBoard().setStone(game.getBoard().getSize() - 1, 2, GoColor.WHITE);
		game.getBoard().setStone(game.getBoard().getSize() - 1, 3, GoColor.BLACK);

		IGroup g1 = game.getBoard().collectGroup(game.getBoard().getStone(game.getBoard().getSize() - 1, 2));

		if (game.getBoard().isGroupSurrounded(g1))
				game.getBoard().removeGroup(g1);

		assertTrue ("collectGroup/isGroupSurrounded/removeGroup test failed",
				 (game.getBoard().getStone(game.getBoard().getSize() - 1, 0) == null) &&
				 (game.getBoard().getStone(game.getBoard().getSize() - 1, 1) == null) &&
				 (game.getBoard().getStone(game.getBoard().getSize() - 1, 2) == null));

		assertFalse ("collectGroup/isGroupSurrounded/removeGroup test failed",
					 (game.getBoard().getStone(game.getBoard().getSize() - 2, 0) == null) ||
					 (game.getBoard().getStone(game.getBoard().getSize() - 2, 1) == null) ||
					 (game.getBoard().getStone(game.getBoard().getSize() - 2, 2) == null) ||
					 (game.getBoard().getStone(game.getBoard().getSize() - 2, 3) == null) ||
					 (game.getBoard().getStone(game.getBoard().getSize() - 1, 3) == null));
	}

	public void test17 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		game.getBoard().setStone(game.getBoard().getSize() / 2 - 2, game.getBoard().getSize() / 2 - 2, GoColor.BLACK);
		game.getBoard().setStone(game.getBoard().getSize() / 2 - 2, game.getBoard().getSize() / 2 - 1, GoColor.BLACK);
		game.getBoard().setStone(game.getBoard().getSize() / 2 - 2, game.getBoard().getSize() / 2, GoColor.BLACK);
		game.getBoard().setStone(game.getBoard().getSize() / 2 - 2, game.getBoard().getSize() / 2 + 1, GoColor.BLACK);
		game.getBoard().setStone(game.getBoard().getSize() / 2 - 2, game.getBoard().getSize() / 2 + 2, GoColor.BLACK);

		game.getBoard().setStone(game.getBoard().getSize() / 2 - 1, game.getBoard().getSize() / 2 - 2, GoColor.BLACK);
		game.getBoard().setStone(game.getBoard().getSize() / 2 - 1, game.getBoard().getSize() / 2 - 1, GoColor.WHITE);
		game.getBoard().setStone(game.getBoard().getSize() / 2 - 1, game.getBoard().getSize() / 2, GoColor.WHITE);
		game.getBoard().setStone(game.getBoard().getSize() / 2 - 1, game.getBoard().getSize() / 2 + 1, GoColor.WHITE);
		game.getBoard().setStone(game.getBoard().getSize() / 2 - 1, game.getBoard().getSize() / 2 + 2, GoColor.BLACK);

		game.getBoard().setStone(game.getBoard().getSize() / 2, game.getBoard().getSize() / 2 - 2, GoColor.BLACK);
		game.getBoard().setStone(game.getBoard().getSize() / 2, game.getBoard().getSize() / 2 - 1, GoColor.WHITE);
		game.getBoard().setStone(game.getBoard().getSize() / 2, game.getBoard().getSize() / 2, GoColor.WHITE);
		game.getBoard().setStone(game.getBoard().getSize() / 2, game.getBoard().getSize() / 2 + 1, GoColor.WHITE);
		game.getBoard().setStone(game.getBoard().getSize() / 2, game.getBoard().getSize() / 2 + 2, GoColor.BLACK);

		game.getBoard().setStone(game.getBoard().getSize() / 2 + 1, game.getBoard().getSize() / 2 - 2, GoColor.BLACK);
		game.getBoard().setStone(game.getBoard().getSize() / 2 + 1, game.getBoard().getSize() / 2 - 1, GoColor.WHITE);
		game.getBoard().setStone(game.getBoard().getSize() / 2 + 1, game.getBoard().getSize() / 2, GoColor.WHITE);
		game.getBoard().setStone(game.getBoard().getSize() / 2 + 1, game.getBoard().getSize() / 2 + 1, GoColor.WHITE);
		game.getBoard().setStone(game.getBoard().getSize() / 2 + 1, game.getBoard().getSize() / 2 + 2, GoColor.BLACK);

		game.getBoard().setStone(game.getBoard().getSize() / 2 + 2, game.getBoard().getSize() / 2 - 2, GoColor.BLACK);
		game.getBoard().setStone(game.getBoard().getSize() / 2 + 2, game.getBoard().getSize() / 2 - 1, GoColor.BLACK);
		game.getBoard().setStone(game.getBoard().getSize() / 2 + 2, game.getBoard().getSize() / 2, GoColor.BLACK);
		game.getBoard().setStone(game.getBoard().getSize() / 2 + 2, game.getBoard().getSize() / 2 + 1, GoColor.BLACK);
		game.getBoard().setStone(game.getBoard().getSize() / 2 + 2, game.getBoard().getSize() / 2 + 2, GoColor.BLACK);
		
		IGroup g1 = game.getBoard().collectGroup(game.getBoard().getStone(game.getBoard().getSize() / 2, game.getBoard().getSize() / 2));

		if (game.getBoard().isGroupSurrounded(g1))
				game.getBoard().removeGroup(g1);

		if (game.getBoard().getStone(game.getBoard().getSize() / 2 - 1, game.getBoard().getSize() / 2 - 1) != null ||
			game.getBoard().getStone(game.getBoard().getSize() / 2 - 1, game.getBoard().getSize() / 2) != null ||
			game.getBoard().getStone(game.getBoard().getSize() / 2 - 1, game.getBoard().getSize() / 2 + 1) != null ||
			game.getBoard().getStone(game.getBoard().getSize() / 2,     game.getBoard().getSize() / 2 - 1) != null ||
			game.getBoard().getStone(game.getBoard().getSize() / 2,     game.getBoard().getSize() / 2) != null ||
			game.getBoard().getStone(game.getBoard().getSize() / 2,     game.getBoard().getSize() / 2 + 1) != null ||
			game.getBoard().getStone(game.getBoard().getSize() / 2 + 1, game.getBoard().getSize() / 2 - 1) != null ||
			game.getBoard().getStone(game.getBoard().getSize() / 2 + 1, game.getBoard().getSize() / 2) != null ||
			game.getBoard().getStone(game.getBoard().getSize() / 2 + 1, game.getBoard().getSize() / 2 + 1) != null)	{
			fail ("collectGroup/isGroupSurrounded/removeGroup test failed");
		}
	}



	public void test18 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		game.getBoard().setStone(game.getBoard().getSize() / 2 - 1, game.getBoard().getSize() / 2, GoColor.WHITE);

		game.getBoard().setStone(game.getBoard().getSize() / 2, game.getBoard().getSize() / 2 - 1, GoColor.WHITE);
		game.getBoard().setStone(game.getBoard().getSize() / 2, game.getBoard().getSize() / 2, GoColor.BLACK);
		game.getBoard().setStone(game.getBoard().getSize() / 2, game.getBoard().getSize() / 2 + 1, GoColor.WHITE);

		game.getBoard().setStone(game.getBoard().getSize() / 2 + 1, game.getBoard().getSize() / 2, GoColor.WHITE);

		IGroup g1 = game.getBoard().collectGroup(game.getBoard().getStone(game.getBoard().getSize() / 2, game.getBoard().getSize() / 2));

		if (game.getBoard().isGroupSurrounded(g1))
				game.getBoard().removeGroup(g1);

		assertNull ("collectGroup/isGroupSurrounded/removeGroup test failed",
					 game.getBoard().getStone(game.getBoard().getSize() / 2, 
							 		game.getBoard().getSize() / 2));
	}
	
	public void test19 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		game.getBoard().setStone(2, 2, GoColor.WHITE);
		game.getBoard().setStone(2, 3, GoColor.WHITE);
		game.getBoard().setStone(2, 4, GoColor.BLACK);
		game.getBoard().setStone(2, 5, GoColor.BLACK);
		game.getBoard().setStone(2, 6, GoColor.BLACK);

		game.getBoard().setStone(3, 1, GoColor.WHITE);
		game.getBoard().setStone(3, 3, GoColor.WHITE);
		game.getBoard().setStone(3, 4, GoColor.WHITE);
		game.getBoard().setStone(3, 5, GoColor.WHITE);
		game.getBoard().setStone(3, 6, GoColor.BLACK);
		game.getBoard().setStone(3, 15, GoColor.BLACK);
		
		game.getBoard().setStone(4, 2, GoColor.WHITE);
		game.getBoard().setStone(4, 3, GoColor.WHITE);
		game.getBoard().setStone(4, 4, GoColor.BLACK);
		game.getBoard().setStone(4, 5, GoColor.BLACK);
		game.getBoard().setStone(4, 6, GoColor.WHITE);
		
		game.getBoard().setStone(5, 2, GoColor.WHITE);
		game.getBoard().setStone(5, 3, GoColor.WHITE);
		game.getBoard().setStone(5, 5, GoColor.BLACK);
		game.getBoard().setStone(5, 6, GoColor.WHITE);

		game.getBoard().setStone(6, 2, GoColor.WHITE);
		game.getBoard().setStone(6, 3, GoColor.BLACK);
		game.getBoard().setStone(6, 4, GoColor.WHITE);
		game.getBoard().setStone(6, 5, GoColor.WHITE);
		game.getBoard().setStone(6, 6, GoColor.BLACK);
		game.getBoard().setStone(6, 7, GoColor.WHITE);
		
		game.getBoard().setStone(7, 2, GoColor.WHITE);
		game.getBoard().setStone(7, 3, GoColor.BLACK);
		game.getBoard().setStone(7, 4, GoColor.BLACK);
		game.getBoard().setStone(7, 5, GoColor.BLACK);
		game.getBoard().setStone(7, 6, GoColor.BLACK);
		game.getBoard().setStone(7, 7, GoColor.WHITE);
		
		game.getBoard().setStone(8, 2, GoColor.WHITE);
		game.getBoard().setStone(8, 4, GoColor.WHITE);
		game.getBoard().setStone(8, 6, GoColor.BLACK);
		
		assertTrue("canDoMove test failed",
					game.canDoMove(5, 4));
	}

	public void test20 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		game.getBoard().setStone(1, 3, GoColor.WHITE);
		game.getBoard().setStone(1, 6, GoColor.WHITE);
		game.getBoard().setStone(1, 7, GoColor.WHITE);
		
		game.getBoard().setStone(2, 2, GoColor.WHITE);
		game.getBoard().setStone(2, 4, GoColor.WHITE);
		game.getBoard().setStone(2, 5, GoColor.WHITE);
		game.getBoard().setStone(2, 6, GoColor.BLACK);
		game.getBoard().setStone(2, 7, GoColor.BLACK);
		game.getBoard().setStone(2, 8, GoColor.WHITE);

		game.getBoard().setStone(3, 2, GoColor.WHITE);
		game.getBoard().setStone(3, 3, GoColor.BLACK);
		game.getBoard().setStone(3, 4, GoColor.WHITE);
		game.getBoard().setStone(3, 5, GoColor.BLACK);
		game.getBoard().setStone(3, 6, GoColor.WHITE);
		game.getBoard().setStone(3, 7, GoColor.WHITE);
		game.getBoard().setStone(3, 8, GoColor.BLACK);
		game.getBoard().setStone(3, 9, GoColor.WHITE);
		game.getBoard().setStone(3, 11, GoColor.BLACK);

		game.getBoard().setStone(4, 2, GoColor.BLACK);
		game.getBoard().setStone(4, 3, GoColor.WHITE);
		game.getBoard().setStone(4, 4, GoColor.BLACK);
		game.getBoard().setStone(4, 5, GoColor.BLACK);
		game.getBoard().setStone(4, 7, GoColor.BLACK);
		game.getBoard().setStone(4, 8, GoColor.WHITE);
		game.getBoard().setStone(4, 14, GoColor.BLACK);

		game.getBoard().setStone(5, 2, GoColor.BLACK);
		game.getBoard().setStone(5, 3, GoColor.WHITE);
		game.getBoard().setStone(5, 4, GoColor.BLACK);
		game.getBoard().setStone(5, 5, GoColor.BLACK);
		game.getBoard().setStone(5, 6, GoColor.BLACK);
		game.getBoard().setStone(5, 7, GoColor.BLACK);
		game.getBoard().setStone(5, 8, GoColor.BLACK);
		game.getBoard().setStone(5, 9, GoColor.WHITE);
		game.getBoard().setStone(5, 15, GoColor.BLACK);

		game.getBoard().setStone(6, 2, GoColor.BLACK);
		game.getBoard().setStone(6, 3, GoColor.BLACK);
		game.getBoard().setStone(6, 4, GoColor.WHITE);
		game.getBoard().setStone(6, 5, GoColor.WHITE);
		game.getBoard().setStone(6, 6, GoColor.BLACK);
		game.getBoard().setStone(6, 7, GoColor.BLACK);
		game.getBoard().setStone(6, 8, GoColor.BLACK);
		game.getBoard().setStone(6, 9, GoColor.WHITE);
		game.getBoard().setStone(6, 15, GoColor.BLACK);

		game.getBoard().setStone(7, 6, GoColor.WHITE);
		game.getBoard().setStone(7, 7, GoColor.WHITE);
		game.getBoard().setStone(7, 8, GoColor.WHITE);

		game.getBoard().setStone(15, 3, GoColor.BLACK);

		assertTrue("canDoMove test failed",
					game.canDoMove(4, 6));

		game.changeCurrentPlayer();

		assertTrue("canDoMove test failed",
					 game.canDoMove(4, 6));
	}

	public void test21 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		game.getBoard().setStone(1, 2, GoColor.WHITE);
		game.getBoard().setStone(1, 3, GoColor.WHITE);
		game.getBoard().setStone(1, 4, GoColor.WHITE);
		
		game.getBoard().setStone(2, 1, GoColor.WHITE);
		game.getBoard().setStone(2, 2, GoColor.BLACK);
		game.getBoard().setStone(2, 3, GoColor.BLACK);
		game.getBoard().setStone(2, 4, GoColor.BLACK);
		game.getBoard().setStone(2, 5, GoColor.WHITE);

		game.getBoard().setStone(3, 1, GoColor.WHITE);
		game.getBoard().setStone(3, 2, GoColor.BLACK);
		game.getBoard().setStone(3, 4, GoColor.BLACK);
		game.getBoard().setStone(3, 5, GoColor.WHITE);

		game.getBoard().setStone(4, 1, GoColor.WHITE);
		game.getBoard().setStone(4, 2, GoColor.BLACK);
		game.getBoard().setStone(4, 3, GoColor.BLACK);
		game.getBoard().setStone(4, 4, GoColor.BLACK);
		game.getBoard().setStone(4, 5, GoColor.WHITE);

		game.getBoard().setStone(5, 2, GoColor.WHITE);
		game.getBoard().setStone(5, 3, GoColor.WHITE);
		game.getBoard().setStone(5, 4, GoColor.WHITE);

		game.changeCurrentPlayer();

		assertTrue("canDoMove test failed",
					game.canDoMove(3, 3));
	}

	public void test22 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		game.getBoard().setStone(1, 2, GoColor.WHITE);
		game.getBoard().setStone(1, 3, GoColor.WHITE);
		game.getBoard().setStone(1, 4, GoColor.WHITE);
		game.getBoard().setStone(1, 5, GoColor.WHITE);
		
		game.getBoard().setStone(2, 1, GoColor.WHITE);
		game.getBoard().setStone(2, 2, GoColor.BLACK);
		game.getBoard().setStone(2, 3, GoColor.BLACK);
		game.getBoard().setStone(2, 4, GoColor.BLACK);
		game.getBoard().setStone(2, 5, GoColor.BLACK);
		game.getBoard().setStone(2, 6, GoColor.WHITE);

		game.getBoard().setStone(3, 1, GoColor.WHITE);
		game.getBoard().setStone(3, 2, GoColor.BLACK);
		game.getBoard().setStone(3, 3, GoColor.BLACK);
		game.getBoard().setStone(3, 4, GoColor.BLACK);
		game.getBoard().setStone(3, 5, GoColor.BLACK);
		game.getBoard().setStone(3, 6, GoColor.WHITE);

		game.getBoard().setStone(4, 1, GoColor.WHITE);
		game.getBoard().setStone(4, 2, GoColor.BLACK);
		game.getBoard().setStone(4, 3, GoColor.BLACK);
		game.getBoard().setStone(4, 5, GoColor.BLACK);
		game.getBoard().setStone(4, 6, GoColor.WHITE);

		game.getBoard().setStone(5, 1, GoColor.WHITE);
		game.getBoard().setStone(5, 2, GoColor.BLACK);
		game.getBoard().setStone(5, 3, GoColor.BLACK);
		game.getBoard().setStone(5, 4, GoColor.BLACK);
		game.getBoard().setStone(5, 5, GoColor.BLACK);
		game.getBoard().setStone(5, 6, GoColor.WHITE);
		
		game.getBoard().setStone(6, 2, GoColor.WHITE);
		game.getBoard().setStone(6, 3, GoColor.WHITE);
		game.getBoard().setStone(6, 4, GoColor.WHITE);
		game.getBoard().setStone(6, 5, GoColor.WHITE);

		assertFalse("canDoMove test failed",
					game.canDoMove(4, 4));
		
		game.changeCurrentPlayer();

		assertTrue("canDoMove test failed",
					 game.canDoMove(4, 4));
	}

	public void test23 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		game.getBoard().setStone(0, 3, GoColor.BLACK);

		game.getBoard().setStone(1, 2, GoColor.BLACK);
		game.getBoard().setStone(1, 3, GoColor.WHITE);
		game.getBoard().setStone(1, 4, GoColor.BLACK);

		game.getBoard().setStone(2, 1, GoColor.BLACK);
		game.getBoard().setStone(2, 3, GoColor.BLACK);

		game.getBoard().setStone(3, 0, GoColor.BLACK);
		game.getBoard().setStone(3, 1, GoColor.WHITE);
		game.getBoard().setStone(3, 2, GoColor.BLACK);

		game.getBoard().setStone(4, 1, GoColor.BLACK);
		
		assertTrue ("canDoMove test failed",
					 game.canDoMove(2, 2));

		game.changeCurrentPlayer();

		assertFalse ("canDoMove test failed",
				 	  game.canDoMove(2, 2));
	}

	public void test24 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		game.getBoard().setStone(0, 3, GoColor.BLACK);

		game.getBoard().setStone(1, 2, GoColor.BLACK);
		game.getBoard().setStone(1, 3, GoColor.WHITE);
		game.getBoard().setStone(1, 4, GoColor.BLACK);

		game.getBoard().setStone(2, 1, GoColor.BLACK);
		game.getBoard().setStone(2, 3, GoColor.BLACK);

		game.getBoard().setStone(3, 0, GoColor.BLACK);
		game.getBoard().setStone(3, 1, GoColor.WHITE);
		game.getBoard().setStone(3, 2, GoColor.BLACK);

		game.getBoard().setStone(4, 1, GoColor.BLACK);
		
		IGroup g1 = game.getBoard().collectGroup(game.getBoard().getStone(1, 3));
		IGroup g2 = game.getBoard().collectGroup(game.getBoard().getStone(3, 1));

		assertTrue ("collectGroup/isGroupSurrounded test failed",
					 game.getBoard().isGroupSurrounded (g1) &&
					 game.getBoard().isGroupSurrounded (g2));
	}

	public void test25 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		game.getBoard().setStone(0, 3, GoColor.BLACK);

		game.getBoard().setStone(1, 2, GoColor.BLACK);
		game.getBoard().setStone(1, 3, GoColor.WHITE);
		game.getBoard().setStone(1, 4, GoColor.BLACK);

		game.getBoard().setStone(2, 1, GoColor.BLACK);
		game.getBoard().setStone(2, 3, GoColor.BLACK);

		game.getBoard().setStone(3, 0, GoColor.BLACK);
		game.getBoard().setStone(3, 1, GoColor.WHITE);
		game.getBoard().setStone(3, 2, GoColor.BLACK);

		game.getBoard().setStone(4, 1, GoColor.BLACK);
		
		IGroup g1 = game.getBoard().collectGroup(game.getBoard().getStone(1, 3));

		if (game.getBoard().isGroupSurrounded (g1)) {
			game.getBoard().removeGroup(g1);			
		}

		g1 = game.getBoard().collectGroup(game.getBoard().getStone(3, 1));
		
		if (game.getBoard().isGroupSurrounded (g1)) {
			game.getBoard().removeGroup(g1);			
		}
		
		assertTrue ("collectGroup/isGroupSurrounded/removeGroup test failed",
					 game.getBoard().getStone(1, 3) == null &&
					 game.getBoard().getStone(3, 1) == null);
	}

	public void test26 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		game.getBoard().setStone(1, 2, GoColor.WHITE);
		game.getBoard().setStone(1, 3, GoColor.WHITE);
		game.getBoard().setStone(1, 4, GoColor.WHITE);

		game.getBoard().setStone(2, 1, GoColor.WHITE);
		game.getBoard().setStone(2, 2, GoColor.BLACK);
		game.getBoard().setStone(2, 3, GoColor.BLACK);
		game.getBoard().setStone(2, 4, GoColor.BLACK);
		game.getBoard().setStone(2, 5, GoColor.WHITE);

		game.getBoard().setStone(3, 1, GoColor.WHITE);
		game.getBoard().setStone(3, 2, GoColor.BLACK);
		game.getBoard().setStone(3, 3, GoColor.WHITE);
		game.getBoard().setStone(3, 5, GoColor.WHITE);

		game.getBoard().setStone(4, 1, GoColor.WHITE);
		game.getBoard().setStone(4, 2, GoColor.BLACK);
		game.getBoard().setStone(4, 3, GoColor.BLACK);
		game.getBoard().setStone(4, 4, GoColor.BLACK);
		game.getBoard().setStone(4, 5, GoColor.WHITE);

		game.getBoard().setStone(5, 2, GoColor.WHITE);
		game.getBoard().setStone(5, 3, GoColor.WHITE);
		game.getBoard().setStone(5, 4, GoColor.WHITE);
		
		assertTrue ("canDoMove test failed",
				     game.canDoMove(3, 4));

		game.changeCurrentPlayer();

		assertTrue ("canDoMove test failed",
			     game.canDoMove(3, 4));
	}

	public void test27 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		for (int i = 0; i < game.getBoard().getSize(); ++i) {
			for (int j = 0; j < game.getBoard().getSize(); ++j) {
				game.getBoard().setStone(i, j, GoColor.BLACK);
			}
		}
		
		IGroup g1 = game.getBoard().collectGroup(game.getBoard().getStone(1, 1));
		
		assertTrue ("collectGroup/isGroupSurrounded test failed",
					 game.getBoard().isGroupSurrounded(g1));
		
	}
	
	public void test28 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);

		game.getBoard().setStone(1, 2, GoColor.BLACK);
		game.getBoard().setStone(2, 1, GoColor.BLACK);
		game.getBoard().setStone(2, 2, GoColor.WHITE);
		game.getBoard().setStone(2, 3, GoColor.BLACK);
		game.getBoard().setStone(3, 1, GoColor.WHITE);
		game.getBoard().setStone(3, 3, GoColor.WHITE);
		game.getBoard().setStone(4, 2, GoColor.WHITE);
		
		assertTrue ("canDoMove test failed", game.canDoMove(3, 2));
		
		game.doMove(3, 2);

		assertFalse("canDoMove test for KO-position failed", game.canDoMove(2, 2));			
	}
	
	public void test29 () {		
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		assertFalse ("canUnduMove test failed", game.canUndoMove());
		
		assertTrue ("canDoMove test failed", game.canDoMove(2, 3));
		
		game.doMove(3, 2);
		
		assertTrue ("canUndoMove test failed", game.canUndoMove());		
	}
	
	public void test30 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);

		assertFalse ("canRedoMove test failed", game.canRedoMove());
		
		assertFalse ("canUndoMove test failed", game.canUndoMove());
		
		assertTrue ("canDoMove test failed", game.canDoMove(6, 6));
		
		game.doMove(6, 6);
		
		assertFalse ("canRedoMove test failed", game.canRedoMove());

		assertTrue ("canUndoMove testFailed", game.canUndoMove());
		
		game.undoMove();

		assertFalse ("canUndoMove testFailed", game.canUndoMove());
		
		assertTrue ("canRedoMove test failed", game.canRedoMove());
		
		game.redoMove();
		
		assertFalse ("canRedoMove test failed", game.canRedoMove());
		
		assertTrue ("canUndoMove test failed", game.canUndoMove());
		
		game.undoMove();
	}

	public void test31 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		assertTrue ("canDoMove test failed", game.canDoMove(4, 5));
		game.doMove(4, 5);
		
		assertTrue ("canDoMove test failed", game.canDoMove(4, 6));
		game.doMove(4, 6);

		assertTrue ("canDoMove test failed", game.canDoMove(4, 7));
		game.doMove(4, 7);
		
		assertFalse ("canRedoMove test failed", game.canRedoMove());		
		
		assertTrue ("canUndoMove test failed", game.canUndoMove());
		
		game.undoMove();

		assertNull (game.getBoard().getStone(4, 7));

		assertTrue ("canRedoMove test failed", game.canRedoMove());
		
		game.redoMove();
		
		assertNotNull (game.getBoard().getStone(4, 5));
		assertNotNull (game.getBoard().getStone(4, 6));
		assertNotNull (game.getBoard().getStone(4, 7));
		
		game.undoMove();
		game.undoMove();
		game.undoMove();

		assertNull (game.getBoard().getStone(4, 5));
		assertNull (game.getBoard().getStone(4, 6));
		assertNull (game.getBoard().getStone(4, 7));
	}

	public void test32 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		game.getBoard().setStone(0, 1, GoColor.BLACK);
		game.getBoard().setStone(0, 2, GoColor.BLACK);
		game.getBoard().setStone(1, 0, GoColor.BLACK);
		game.getBoard().setStone(1, 1, GoColor.WHITE);
		game.getBoard().setStone(1, 2, GoColor.WHITE);
		game.getBoard().setStone(2, 1, GoColor.BLACK);
		game.getBoard().setStone(2, 2, GoColor.BLACK);
		
		game.doMove(1, 3);
		
		assertTrue ("doMove test failed",
					(game.getBoard().getStone(1, 1) == null) &&
					(game.getBoard().getStone(1, 2) == null) &&
					(game.getBoard().getStone(1, 3) != null));
		
		game.undoMove();

		assertTrue ("undoMove test failed",
				(game.getBoard().getStone(1, 1) != null) &&
				(game.getBoard().getStone(1, 2) != null) &&
				(game.getBoard().getStone(1, 3) == null));
		
		game.redoMove();

		assertTrue ("doMove test failed",
				(game.getBoard().getStone(1, 1) == null) &&
				(game.getBoard().getStone(1, 2) == null) &&
				(game.getBoard().getStone(1, 3) != null));
	}

	public void test33 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		BoardPoint point;		
		
		for (int i = 0; i < game.getBoard().getSize(); ++i) {
			for (int j = 0; j < game.getBoard().getSize(); ++j) {
				point = new BoardPoint (game.getBoard(), i, j);
				assertTrue ("BoardPoint constructor with parameters failed", point.isValid());
			}
		}
		
		point = new BoardPoint ();
		assertFalse ("BoardPoint constructor without parameters failed", point.isValid());
	}

	public void test34 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		game.doMove(1, 1);
		
		assertTrue ("canUndoMove test failed", game.canUndoMove());
		
		game.clear();
	
		assertFalse ("permitUndoMove/canUndoMove test failed", game.canUndoMove());		
	}

	public void test35 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		game.doMove(1, 1);
		
		assertTrue ("canUndoMove test failed", game.canUndoMove());
		
		game.undoMove();
		
		assertTrue ("canRedoMove test failed", game.canRedoMove());
		
		game.clear();

		assertFalse ("permitRedoMove/canRedoMove test failed", game.canRedoMove());
	}

	public void test36 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		game.getBoard().setStone(1, 2, GoColor.BLACK);
		game.getBoard().setStone(2, 1, GoColor.BLACK);
		game.getBoard().setStone(2, 2, GoColor.WHITE);
		game.getBoard().setStone(2, 3, GoColor.BLACK);
		game.getBoard().setStone(3, 1, GoColor.WHITE);
		game.getBoard().setStone(3, 3, GoColor.WHITE);
		game.getBoard().setStone(4, 2, GoColor.WHITE);
		
		assertTrue ("canDoMove test failed", game.canDoMove(3, 2));
		
		game.doMove(3, 2);

		assertFalse("canDoMove test for KO-position failed", game.canDoMove(2, 2));
		
		game.doMove(10, 10);
		
		game.undoMove();
		
		assertFalse("canDoMove test for KO-position failed", game.canDoMove(2, 2));
	}

	public void test37 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		assertFalse ("canPass test failed", game.canPass());
		assertFalse ("isGameOver test failed", game.isGameOver());
		
		try {
			game.pass();
		} catch (GoException ex) {			
		}
		
		game.doMove(1, 1);
		assertTrue ("canPass test failed", game.canPass());
		assertFalse ("isGameOver test failed", game.isGameOver());

		game.doMove(2, 2);
		assertTrue ("canPass test failed", game.canPass());
		assertFalse ("isGameOver test failed", game.isGameOver());
		
		assertFalse ("canDoMove test failed", game.canDoMove(2, 2));

		try {
			game.doMove(2, 2);
		} catch (GoException ex) {			
		}
		
		game.pass();
		game.pass();
		
		assertFalse ("canPass test failed", game.canPass());
		assertTrue ("isGameOver test failed", game.isGameOver());		
	}
	
	public void test38 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		assertFalse("modified merhod rest failed", game.modified());
		game.doMove(1, 1);
		assertTrue("modified merhod rest failed", game.modified());		
		game.undoMove();
		assertTrue("modified merhod rest failed", game.modified());
		game.justSaved();
		assertFalse("modified merhod rest failed", game.modified());
	}

	public void test39 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		assertTrue("wrong game buffer index", game.getBufferCurrentIndex() == 0);
		assertTrue("wrong game buffer", game.getGameBuffer().isEmpty());
		game.doMove(5, 5);
		assertFalse("wrong game buffer", game.getGameBuffer().isEmpty());
		assertFalse("wrong game buffer index", game.getBufferCurrentIndex() == 0);
		
		game.doMove(1, 1);
		game.doMove(1, 2);
		game.doMove(2, 2);
		game.doMove(2, 1);
		assertTrue("wrong game buffer index", game.getBufferCurrentIndex() == 5);
	}

	public void test40 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		game.doMove(14, 14);
		game.pass();
		assertTrue ("canDoMove for Ko position with pass failed", game.canDoMove(1, 12));
	}
}

