package org.amse.byBedrosova.tests;

import java.io.*;

import junit.framework.TestCase;

import org.amse.byBedrosova.go.io.*;
import org.amse.byBedrosova.go.model.IGame;
import org.amse.byBedrosova.go.model.impl.ModelFactory;
import org.amse.byBedrosova.go.exception.*;

public class WriterTests extends TestCase {
	private static final String myPath = "testing/templates/";
	
/*comparator test*/
	public void test0 () {
		fileComparator x = new fileComparator ();
		
		assertTrue ("Compareing file with itself failed", x.areEqual(myPath + "template0.go", myPath + "template0.go"));
	}	
/*comparator test*/
	public void test1 () {
		fileComparator x = new fileComparator ();
		
		assertTrue ("Compareing file with it's copy failed", x.areEqual(myPath + "template0.go", myPath + "template1.go"));
	}	

/*ok file with empty game*/
	public void test2 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
			
		try {
			(new GoWriter (game)).saveGame(myPath + "test2.go");				
		} catch (GoException ex) {
			fail ("writer failed");
		}

		fileComparator x = new fileComparator ();
		
		assertTrue ("Compareing with right file failed", x.areEqual(myPath + "test2.go", myPath + "template2.go"));
	}	

/*file with simple position*/	
	public void test3 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		game.doMove(5, 5);
			
		try {
			(new GoWriter (game)).saveGame(myPath + "test3.go");				
		} catch (GoException ex) {
			fail ("writer failed");
		}

		fileComparator x = new fileComparator ();
		
		assertTrue ("Compareing with right file failed", x.areEqual(myPath + "test3.go", myPath + "template3.go"));
		assertFalse ("Compareing with wrong file failed", x.areEqual(myPath + "test3.go", myPath + "template1.go"));
	}	

/*file with simple position*/	
	public void test4 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		game.doMove(5, 5);
		game.doMove(5, 15);
			
		try {
			(new GoWriter (game)).saveGame(myPath + "test4.go");				
		} catch (GoException ex) {
			fail ("writer failed");
		}

		fileComparator x = new fileComparator ();
		
		assertFalse ("Compareing with wrong file", x.areEqual(myPath + "test4.go", myPath + "template3.go"));
	}	
/*read/write tst*/	
	public void test5 () {
		ModelFactory gf = new ModelFactory ();
		IGame game = gf.newGame(19);
		
		game.doMove(1, 2);
		game.doMove(2, 2);
		game.doMove(2, 1);
		game.doMove(3, 1);
		game.doMove(2, 3);
		game.doMove(3, 3);
		game.doMove(18, 18);
		game.doMove(4, 2);
			
		try {
			(new GoWriter (game)).saveGame(myPath + "test5.go");				
		} catch (GoException ex) {
			fail ("writer failed");
		}
		
		IGame tmpGame = gf.newGame(19);

		try {
			(new GoReader (tmpGame)).loadGame(myPath + "test5.go");				
		} catch (GoException ex) {
			fail ("reader failed");
		}

		for (int i = 0; i < game.getBoard().getSize(); ++i) {
			for (int j = 0; j < game.getBoard().getSize(); ++j) {
				if (game.getBoard().getStone(i, j) == null) {
					if (tmpGame.getBoard().getStone(i, j) != null) {
						fail ("Big test failed");
					}
				} else {
					assertTrue("Big test failed", game.getBoard().getStone(i, j).equals(tmpGame.getBoard().getStone(i, j)));
				}
			}
		}
	}	
	private class fileComparator {
		public boolean areEqual (String path1, String path2) {
			Reader r1;
			Reader r2;
			try {
				r1 = new BufferedReader (new FileReader (path1));
				r2 = new BufferedReader (new FileReader (path2));
				
				int c1, c2;
				
				while (((c1 = r1.read ()) != -1) & ((c2 = r2.read()) != -1)) {
					if (c1 != c2) {
						return false;
					}
				}
				
				if (c1 == c2) {
					return true;
				}
			} catch (FileNotFoundException e) {
				e.printStackTrace();
				return false;
			} catch (IOException e) {			
				e.printStackTrace();
				return false;				
			}			
			return false;
		}
	}
}

